@@ -1,13 +1,18 @@
 """
 .. include:: ./documentation.md
 """
+from .serie import *
+from .data import *
+from .dataset import *
+
 from .model import *
 from .sm import *
 from .mosm import *
 from .csm import *
 from .conv import *
 from .sm_lmc import *
-from .data import *
-from .dataset import *
+
 from .plot import *
 from .errors import *
+
+from .kernels.config import *
