@@ -12,6 +12,7 @@ class CONV(Model):
         dataset (mogptk.dataset.DataSet): `DataSet` object of data for all channels.
         Q (int): Number of components.
         model: Gaussian process model to use, such as `mogptk.model.Exact`.
+        mean (mogptk.kernels.mean.Mean): The mean class.
         name (str): Name of the model.
 
     Attributes:
@@ -37,7 +38,7 @@ class CONV(Model):
 
     [1] M.A. Álvarez and N.D. Lawrence, "Sparse Convolved Multiple Output Gaussian Processes", Advances in Neural Information Processing Systems 21, 2009
     """
-    def __init__(self, dataset, Q=1, model=Exact(), name="CONV"):
+    def __init__(self, dataset, Q=1, model=Exact(), mean=None, name="CONV"):
         if not isinstance(dataset, DataSet):
             dataset = DataSet(dataset)
         dataset.rescale_x()
@@ -53,7 +54,7 @@ class CONV(Model):
         )
         kernel = MixtureKernel(conv, Q)
 
-        super(CONV, self).__init__(dataset, kernel, model, name=name)
+        super(CONV, self).__init__(dataset, kernel, model, mean, name)
         self.Q = Q
 
     def init_parameters(self, method='SM', sm_init='BNSE', sm_method='Adam', sm_iters=100, sm_params={}, sm_plot=False):
