@@ -1,24 +1,27 @@
+import copy
+
 import numpy as np
 import pandas as pd
-from .data import *
 import matplotlib.pyplot as plt
 
-def LoadCSV(filename, x_col=0, y_col=1, name=None, formats={}, **kwargs):
+from .data import Data
+
+def LoadCSV(filename, x_col=0, y_col=1, name=None, **kwargs):
     """
-    LoadCSV loads a dataset from a given CSV file. It loads in x_cols as the names of the input dimension columns, and y_cols the name of the output columns. Setting a formatter for a column will enable parsing for example date fields such as '2019-03-01'. A filter can be set to filter out data from the CSV, such as ensuring that another column has a certain value.
+    LoadCSV loads a dataset from a given CSV file. It loads in `x_col` as the names of the input dimension columns, and `y_col` as the names of the output columns.
+
     Args:
         filename (str): CSV filename.
         x_col (int, str, list of int or str): Names or indices of X column(s) in CSV.
         y_col (int, str, list of int or str): Names or indices of Y column(s) in CSV.
-        name (str, list, optional): Name or names of data channels.
-        formats (dict, optional): Dictionary with x_col values as keys containing FormatNumber (default), FormatDate, FormetDateTime, ...
+        name (str, list): Name or names of data channels.
         **kwargs: Additional keyword arguments for csv.DictReader.
 
     Returns:
         mogptk.data.Data or mogptk.dataset.DataSet
 
     Examples:
-        >>> LoadCSV('gold.csv', 'Date', 'Price', name='Gold', formats={'Date': FormatDate})
+        >>> LoadCSV('gold.csv', 'Date', 'Price', name='Gold')
         <mogptk.dataset.DataSet at ...>
         >>> LoadCSV('gold.csv', 'Date', 'Price', sep=' ', quotechar='|')
         <mogptk.dataset.DataSet at ...>
@@ -26,18 +29,17 @@ def LoadCSV(filename, x_col=0, y_col=1, name=None, formats={}, **kwargs):
 
     df = pd.read_csv(filename, **kwargs)
 
-    return LoadDataFrame(df, x_col, y_col, name, formats)
+    return LoadDataFrame(df, x_col, y_col, name)
 
-def LoadDataFrame(df, x_col=0, y_col=1, name=None, formats={}):
+def LoadDataFrame(df, x_col=0, y_col=1, name=None):
     """
-    LoadDataFrame loads a DataFrame from Pandas. It loads in x_cols as the names of the input dimension columns, and y_cols the names of the output columns. Setting a formatter for a column will enable parsing for example date fields such as '2019-03-01'. A filter can be set to filter out data from the CSV, such as ensuring that another column has a certain value.
+    LoadDataFrame loads a DataFrame from pandas. It loads in `x_col` as the names of the input dimension columns, and `y_col` the names of the output columns.
 
     Args:
-        df (pandas.DataFrame): The Pandas DataFrame.
+        df (pandas.DataFrame): The pandas DataFrame.
         x_col (int, str, list of int or str): Names or indices of X column(s) in DataFrame.
         y_col (int, str, list of int or str): Names or indices of Y column(s) in DataFrame.
-        name (str, list of str, optional): Name or names of data channels.
-        formats (dict, optional): Dictionary with x_col values as keys containing FormatNumber (default), FormatDate, FormetDateTime, ...
+        name (str, list of str): Name or names of data channels.
 
     Returns:
         mogptk.data.Data or mogptk.dataset.DataSet
@@ -58,7 +60,7 @@ def LoadDataFrame(df, x_col=0, y_col=1, name=None, formats={}):
     if not isinstance(y_col, list):
         y_col = [y_col]
 
-    if name == None:
+    if name is None:
         name = [None] * len(y_col)
     else:
         if not isinstance(name, list):
@@ -75,34 +77,11 @@ def LoadDataFrame(df, x_col=0, y_col=1, name=None, formats={}):
     df = df[x_col + y_col]
     if len(df.index) == 0:
         raise ValueError("dataframe cannot be empty")
-    #df = df.dropna()
-    #if len(df.index) == 0:
-    #    raise ValueError("dataframe has NaN values for every row, consider selecting X and Y columns that have valid values by setting x_col and y_col")
 
     input_dims = len(x_col)
     x_data = df[x_col]
     x_labels = [str(item) for item in x_col]
 
-    # set formatters automatically if not already set, try and see if we can parse datetime values
-    for col in df.columns:
-        if col not in formats:
-            dtype = df.dtypes[col]
-            if np.issubdtype(dtype, np.number):
-                formats[col] = FormatNumber()
-            elif np.issubdtype(dtype, np.datetime64):
-                formats[col] = FormatDateTime()
-            elif np.issubdtype(dtype, np.object_):
-                first = df[col].iloc[0]
-                try:
-                    _ = float(first)
-                    formats[col] = FormatNumber()
-                except:
-                    try:
-                        _ = dateutil.parser.parse(first)
-                        formats[col] = FormatDateTime()
-                    except:
-                        raise ValueError("unknown format for column %s, must be a number type or datetime" % (col,))
-
     dataset = DataSet()
     for i in range(len(y_col)):
         channel = df[x_col + [y_col[i]]].dropna()
@@ -111,7 +90,6 @@ def LoadDataFrame(df, x_col=0, y_col=1, name=None, formats={}):
             channel[x_col].values,
             channel[y_col[i]].values,
             name=name[i],
-            formats=formats,
             x_labels=x_labels,
             y_label=str(y_col[i]),
         ))
@@ -125,22 +103,51 @@ def LoadDataFrame(df, x_col=0, y_col=1, name=None, formats={}):
 
 class DataSet:
     """
-        DataSet is a class that holds multiple Data objects as channels.
+    DataSet is a class that holds multiple Data objects as channels. It is the complete representation of the data used for fitting multi-output Gaussian processes.
 
-        Args:
-            *args (mogptk.data.Data, mogptk.dataset.DataSet, list, dict): Accepts multiple arguments,
-                each of which should be either a DataSet or Data object, a list of
-                Data objects or a dictionary of Data objects. Each Data object will be added to the
-                list of channels. In case of a dictionary, the key will set the name of the Data object.
-                If a DataSet is passed, its channels will be added.
+    Args:
+        *args (mogptk.data.Data, mogptk.dataset.DataSet, list, dict, numpy.ndarray): Accepts multiple arguments, each of which should be either a `DataSet` or `Data` object, a list of `Data` objects or a dictionary of `Data` objects. Each `Data` object will be added to the list of channels. In case of a dictionary, the key will set the name of the channel. If a `DataSet` is passed its channels will be added. It is also possible to pass X and Y data array directly by either passing two `numpy.ndarrays` or two lists of `numpy.ndarrays` for X and Y data.
 
-        Examples:
-            >>> dataset = mogptk.DataSet(channel_a, channel_b, channel_c)
-        """
-    def __init__(self, *args):
+    Examples:
+        Different ways to initiate a DataSet:
+        >>> wind_velocity = mogptk.LoadDataFrame(df, x_col='Date', y_col='Wind Velocity', name='wind')
+        >>> tidal_height = mogptk.LoadDataFrame(df, x_col='Date', y_col='Tidal Height', name='tidal')
+        >>> dataset = mogptk.DataSet(wind_velocity, tidal_height)
+
+        >>> dataset = mogptk.DataSet(
+        >>>     mogptk.LoadDataFrame(df, x_col='Date', y_col='Wind Velocity', name='wind'),
+        >>>     mogptk.LoadDataFrame(df, x_col='Date', y_col='Tidal Height', name='tidal'),
+        >>> )
+
+        >>> dataset = mogptk.DataSet()
+        >>> dataset.append(mogptk.LoadDataFrame(df, x_col='Date', y_col='Wind Velocity', name='wind'))
+        >>> dataset.append(mogptk.LoadDataFrame(df, x_col='Date', y_col='Tidal Height', name='tidal'))
+
+        >>> dataset = mogptk.DataSet(x, y)
+
+        >>> dataset = mogptk.DataSet(x, [y1, y2, y3], names=['A', 'B', 'C'])
         
+        >>> dataset = mogptk.DataSet([x1, x2, x3], [y1, y2, y3])
 
+        Accessing individual channels:
+        >>> dataset[0]       # first channel
+        >>> dataset['wind']  # wind velocity channel
+    """
+    def __init__(self, *args, names=None):
         self.channels = []
+        if len(args) == 2 and (isinstance(args[1], np.ndarray) or isinstance(args[1], list) and all(isinstance(item, np.ndarray) for item in args[1])):
+            if names is None or isinstance(names, str):
+                names = [names]
+
+            if isinstance(args[0], np.ndarray):
+                for name, y in zip(names, args[1]):
+                    self.append(Data(args[0], y, name=name))
+                return
+            elif isinstance(args[0], list) and all(isinstance(item, np.ndarray) for item in args[0]) and isinstance(args[1], list) and len(args[0]) == len(args[1]):
+                for name, x, y in zip(names, args[0], args[1]):
+                    self.append(Data(x, y, name=name))
+                return
+
         for arg in args:
             self.append(arg)
 
@@ -155,6 +162,14 @@ class DataSet:
             return self.channels[self.get_names().index(key)]
         return self.channels[key]
 
+    def __setitem__(self, key, arg):
+        if isinstance(arg, Data):
+            self.channels[key] = arg
+        elif isinstance(arg, DataSet) and len(arg) == 1:
+            self.channels[key] = arg[0]
+        else:
+            raise ValueError("must set a data type of Data or a DataSet with a single channel")
+
     def __str__(self):
         return self.__repr__()
     
@@ -166,10 +181,10 @@ class DataSet:
 
     def append(self, arg):
         """
-        Append channel(s) to DataSet.
+        Append channel(s) to the DataSet.
         
         Args:
-            arg (mogptk.data.Data, mogptk.dataset.DataSet, list, dict): Argument can be either a DataSet or Data object, a list of Data objects or a dictionary of Data objects. Each Data object will be added to the list of channels. In case of a dictionary, the key will set the name of the Data object. If a DataSet is passed, its channels will be added.
+            arg (mogptk.data.Data, mogptk.dataset.DataSet, list, dict): Argument can be either a `DataSet` or `Data` object, a list of `Data` objects or a dictionary of `Data` objects. Each `Data` object will be added to the list of channels. In case of a dictionary, the key will set the name of the channel. If a `DataSet` is passed, its channels will be added.
 
         Examples:
             >>> dataset.append(mogptk.LoadFunction(lambda x: np.sin(5*x[:,0]), n=200, start=0.0, end=4.0, name='A'))
@@ -187,15 +202,91 @@ class DataSet:
                 val.name = key
                 self.channels.append(val)
         else:
-            raise Exception("unknown data type %s in append to DataSet" % (type(arg)))
+            raise ValueError("unknown data type %s in append to DataSet" % (type(arg)))
         return self
 
+    def copy(self):
+        """
+        Make a deep copy of `DataSet`.
+
+        Returns:
+            mogptk.dataset.DataSet
+
+        Examples:
+            >>> other = dataset.copy()
+        """
+        return copy.deepcopy(self)
+
+    def rescale_x(self, upper=1000.0):
+        """
+        Rescale the X axis so that it is in the interval [0.0,upper] for each channel. This helps training most kernels.
+
+        Args:
+            upper (float): Upper end of the interval.
+
+        Examples:
+            >>> dataset.rescale_x()
+        """
+        for channel in self.channels:
+            channel.rescale_x(upper)
+
+    def transform(self, transformer):
+        """
+        Transform each channel by using one of the provided transformers, such as `TransformDetrend`, `TransformLinear`, `TransformLog`, `TransformNormalize`, `TransformStandard`, etc.
+
+        Args:
+            transformer (obj): Transformer object derived from TransformBase.
+
+        Examples:
+            >>> dataset.transform(mogptk.TransformDetrend(degree=2))        # remove polynomial trend
+            >>> dataset.transform(mogptk.TransformLinear(slope=1, bias=2))  # remove linear trend
+            >>> dataset.transform(mogptk.TransformLog)                      # log transform the data
+            >>> dataset.transform(mogptk.TransformNormalize)                # transform to [-1,1]
+            >>> dataset.transform(mogptk.TransformStandard)                 # transform to mean=0, var=1
+        """
+        for channel in self.channels:
+            channel.transform(transformer)
+
+    def filter(self, start, end):
+        """
+        Filter the data range to be between `start` and `end` in the X axis.
+
+        Args:
+            start (float, str): Start of interval.
+            end (float, str): End of interval.
+
+        Examples:
+            >>> dataset.filter(3, 8)
+
+            >>> dataset.filter('2016-01-15', '2016-06-15')
+        """
+        for channel in self.channels:
+            channel.filter(start, end)
+
+    def aggregate(self, duration, f=np.mean):
+        """
+        Aggregate the data by duration and apply a function to obtain a reduced dataset.
+
+        For example, group daily data by week and take the mean. The duration can be set as a number which defined the intervals on the X axis, or by a string written in the duration format in case the X axis has data type `numpy.datetime64`. The duration format uses: Y=year, M=month, W=week, D=day, h=hour, m=minute, and s=second. For example, 3W1D means three weeks and one day, ie. 22 days, or 6M to mean six months.
+
+        Args:
+            duration (float, str): Duration along the X axis or as a string in the duration format.
+            f (function): Function to use to reduce data.
+
+        Examples:
+            >>> dataset.aggregate(5)
+
+            >>> dataset.aggregate('2W', f=np.sum)
+        """
+        for channel in self.channels:
+            channel.aggregate(duration, f)
+
     def get_input_dims(self):
         """
         Return the input dimensions per channel.
 
         Returns:
-            list: List of input dimensions per channel.
+            list: List of the number of input dimensions per channel.
 
         Examples:
             >>> dataset.get_input_dims()
@@ -208,7 +299,7 @@ class DataSet:
         Return the output dimensions of the dataset, i.e. the number of channels.
 
         Returns:
-            int: Output dimensions.
+            int: Number of output dimensions.
 
         Examples:
             >>> dataset.get_output_dims()
@@ -221,13 +312,13 @@ class DataSet:
         Return the names of the channels.
 
         Returns:
-            list: List of names.
+            list: List of channel names.
 
         Examples:
             >>> dataset.get_names()
             ['A', 'B', 'C']
         """
-        return [channel.get_name() if channel.get_name() != "" else "#"+str(i+1) for i, channel in enumerate(self.channels)]
+        return [channel.get_name() for i, channel in enumerate(self.channels)]
 
     def get(self, index):
         """
@@ -251,23 +342,35 @@ class DataSet:
                     return channel
         raise ValueError("channel '%d' does not exist in DataSet" % (index))
     
-    def get_train_data(self):
+    def get_index(self, index):
         """
-        Returns observations used for training.
+        Return channel's numeric index given its name.
+
+        Args:
+            index (int, str): Index or name of the channel.
 
         Returns:
-            list: X data of shape (n,input_dims) per channel.
-            list: Y data of shape (n,) per channel.
+            int: Channel index.
 
         Examples:
-            >>> x, y = dataset.get_train_data()
+            >>> channel_index = dataset.get_index('A')
         """
-        return [channel.get_train_data()[0] for channel in self.channels], [channel.get_train_data()[1] for channel in self.channels]
+        if isinstance(index, int):
+            if index < len(self.channels):
+                return index
+        elif isinstance(index, str):
+            for channel in self.channels:
+                if channel.name == index:
+                    return index
+        raise ValueError("channel '%d' does not exist in DataSet" % (index))
     
-    def get_data(self):
+    def get_data(self, transformed=False):
         """
         Returns all observations, train and test.
 
+        Arguments:
+            transformed (boolean): Return transformed data.
+
         Returns:
             list: X data of shape (n,input_dims) per channel.
             list: Y data of shape (n,) per channel.
@@ -275,11 +378,30 @@ class DataSet:
         Examples:
             >>> x, y = dataset.get_data()
         """
-        return [channel.get_data()[0] for channel in self.channels], [channel.get_data()[1] for channel in self.channels]
-
-    def get_test_data(self):
+        return [channel.get_data(transformed=transformed)[0] for channel in self.channels], [channel.get_data(transformed=transformed)[1] for channel in self.channels]
+    
+    def get_train_data(self, transformed=False):
         """
-        Returns the observations used for testing.
+        Returns observations used for training.
+
+        Arguments:
+            transformed (boolean): Return transformed data.
+
+        Returns:
+            list: X data of shape (n,input_dims) per channel.
+            list: Y data of shape (n,) per channel.
+
+        Examples:
+            >>> x, y = dataset.get_train_data()
+        """
+        return [channel.get_train_data(transformed=transformed)[0] for channel in self.channels], [channel.get_train_data(transformed=transformed)[1] for channel in self.channels]
+
+    def get_test_data(self, transformed=False):
+        """
+        Returns the observations used for testing which correspond to the removed points.
+
+        Arguments:
+            transformed (boolean): Return transformed data.
 
         Returns:
             list: X data of shape (n,input_dims) per channel.
@@ -288,15 +410,31 @@ class DataSet:
         Examples:
             >>> x, y = dataset.get_test_data()
         """
-        return [channel.get_test_data()[0] for channel in self.channels], [channel.get_test_data()[1] for channel in self.channels]
+        return [channel.get_test_data(transformed=transformed)[0] for channel in self.channels], [channel.get_test_data(transformed=transformed)[1] for channel in self.channels]
     
-    def get_prediction(self, name, sigma=2):
+    def get_prediction_x(self):
         """
-        Returns the prediction of a given name with a normal variance of sigma.
+        Returns the prediction X range for all channels.
+
+        Returns:
+            list: X prediction of shape (n,input_dims) per channel.
+
+        Examples:
+            >>> x = dataset.get_prediction_x()
+        """
+        x = []
+        for channel in self.channels:
+            x.append(channel.get_prediction_x())
+        return x
+    
+    def get_prediction(self, name, sigma=2.0, transformed=False):
+        """
+        Returns the prediction of a given name with a confidence interval of `sigma` times the standard deviation.
 
         Args:
-            name (str): Name of the prediction, equals the name of the model that made the prediction.
-            sigma (float): The uncertainty interval calculated at mean-sigma*var and mean+sigma*var. Defaults to 2,
+            name (str): Name of the model of the prediction.
+            sigma (float): The confidence interval's number of standard deviations.
+            transformed (boolean): Return transformed data as used for training.
 
         Returns:
             list: X prediction of shape (n,input_dims) per channel.
@@ -312,16 +450,16 @@ class DataSet:
         lower = []
         upper = []
         for channel in self.channels:
-            channel_x, channel_mu, channel_lower, channel_upper = channel.get_prediction(name, sigma)
-            x.append(channel_x)
-            mu.append(channel_mu)
-            lower.append(channel_lower)
-            upper.append(channel_upper)
+            cx, cmu, clower, cupper = channel.get_prediction(name, sigma, transformed=transformed)
+            x.append(cx)
+            mu.append(cmu)
+            lower.append(clower)
+            upper.append(cupper)
         return x, mu, lower, upper
 
     def set_prediction_x(self, x):
         """
-        Set the prediction range per channel.
+        Set the prediction range directly for saved predictions per channel. This will clear old predictions.
 
         Args:
             x (list, dict): Array of shape (n,) or (n,input_dims) per channel with prediction X values. If a dictionary is passed, the index is the channel index or name.
@@ -350,8 +488,8 @@ class DataSet:
         Args:
             start (list, dict): Start values for prediction range per channel.
             end (list, dict): End values for prediction range per channel.
-            n (list, dict, optional): Number of points for prediction range per channel.
-            step (list, dict, optional): Step size for prediction range per channel.
+            n (list, dict): Number of points for prediction range per channel.
+            step (list, dict): Step size for prediction range per channel.
 
         Examples:
             >>> dataset.set_prediction_range([2, 3], [5, 6], [4, None], [None, 0.5])
@@ -365,13 +503,13 @@ class DataSet:
             end = [end] * self.get_output_dims()
         elif isinstance(end, dict):
             end = [end[name] for name in self.get_names()]
-        if n == None:
+        if n is None:
             n = [None] * self.get_output_dims()
         elif not isinstance(n, (list, dict)):
             n = [n] * self.get_output_dims()
         elif isinstance(n, dict):
             n = [n[name] for name in self.get_names()]
-        if step == None:
+        if step is None:
             step = [None] * self.get_output_dims()
         elif not isinstance(step, (list, dict)):
             step = [step] * self.get_output_dims()
@@ -383,10 +521,17 @@ class DataSet:
 
         for i, channel in enumerate(self.channels):
             channel.set_prediction_range(start[i], end[i], n[i], step[i])
+
+    def clear_predictions(self):
+        """
+        Clear all saved predictions for all channels.
+        """
+        for i, channel in enumerate(self.channels):
+            channel.clear_predictions()
     
     def get_nyquist_estimation(self):
         """
-        Estimate nyquist frequency by taking 0.5/(minimum distance of points).
+        Estimate the Nyquist frequency by taking 0.5/(minimum distance of points) per channel.
 
         Returns:
             list: Nyquist frequency array of shape (input_dims) per channel.
@@ -396,18 +541,44 @@ class DataSet:
         """
         return [channel.get_nyquist_estimation() for channel in self.channels]
     
-    def get_bnse_estimation(self, Q, n=8000):
+    def get_lombscargle_estimation(self, Q=1, n=10000):
         """
-        Peaks estimation using BNSE (Bayesian Non-parametric Spectral Estimation).
+        Peak estimation of the spectrum using Lomb-Scargle per channel.
 
         Args:
-            Q (int): Number of peaks to find, defaults to 1.
-            n (int): Number of points of the grid to evaluate frequencies, defaults to 5000.
+            Q (int): Number of peaks to find.
+            n (int): Number of points of the grid to evaluate frequencies.
 
         Returns:
-            list: Amplitude array of shape (input_dims,Q) per channel.
-            list: Frequency array of shape (input_dims,Q) per channel.
-            list: Variance array of shape (input_dims,Q) per channel.
+            list: Amplitude array of shape (Q,input_dims) per channel.
+            list: Frequency array of shape (Q,input_dims) per channel.
+            list: Variance array of shape (Q,input_dims) per channel.
+
+        Examples:
+            >>> amplitudes, means, variances = dataset.get_lombscargle_estimation()
+        """
+        amplitudes = []
+        means = []
+        variances = []
+        for channel in self.channels:
+            channel_amplitudes, channel_means, channel_variances = channel.get_lombscargle_estimation(Q, n)
+            amplitudes.append(channel_amplitudes)
+            means.append(channel_means)
+            variances.append(channel_variances)
+        return amplitudes, means, variances
+    
+    def get_bnse_estimation(self, Q=1, n=1000):
+        """
+        Peak estimation of the spectrum using BNSE (Bayesian Non-parametric Spectral Estimation) per channel.
+
+        Args:
+            Q (int): Number of peaks to find.
+            n (int): Number of points of the grid to evaluate frequencies.
+
+        Returns:
+            list: Amplitude array of shape (Q,input_dims) per channel.
+            list: Frequency array of shape (Q,input_dims) per channel.
+            list: Variance array of shape (Q,input_dims) per channel.
 
         Examples:
             >>> amplitudes, means, variances = dataset.get_bnse_estimation()
@@ -422,166 +593,110 @@ class DataSet:
             variances.append(channel_variances)
         return amplitudes, means, variances
     
-    def get_lombscargle_estimation(self, Q, n=50000):
+    def get_sm_estimation(self, Q=1, method='BNSE', optimizer='Adam', iters=100, params={}, plot=False):
         """
-        Peaks estimation using Lomb Scargle.
+        Peak estimation of the spectrum using the spectral mixture kernel per channel.
 
         Args:
-            Q (int): Number of peaks to find, defaults to 1.
-            n (int): Number of points of the grid to evaluate frequencies, defaults to 50000.
+            Q (int): Number of peaks to find.
+            method (str): Method of estimating SM kernels.
+            optimizer (str): Optimization method for SM kernels.
+            iters (str): Maximum iteration for SM kernels.
+            params (object): Additional parameters for PyTorch optimizer.
+            plot (bool): Show the PSD of the kernel after fitting.
 
         Returns:
-            list: Amplitude array of shape (input_dims,Q) per channel.
-            list: Frequency array of shape (input_dims,Q) per channel.
-            list: Variance array of shape (input_dims,Q) per channel.
+            list: Amplitude array of shape (Q,input_dims) per channel.
+            list: Frequency array of shape (Q,input_dims) per channel.
+            list: Variance array of shape (Q,input_dims) per channel.
 
         Examples:
-            >>> amplitudes, means, variances = dataset.get_lombscargle_estimation()
+            >>> amplitudes, means, variances = dataset.get_sm_estimation()
         """
         amplitudes = []
         means = []
         variances = []
         for channel in self.channels:
-            channel_amplitudes, channel_means, channel_variances = channel.get_lombscargle_estimation(Q, n)
+            channel_amplitudes, channel_means, channel_variances = channel.get_sm_estimation(Q, method, optimizer, iters, params, plot)
             amplitudes.append(channel_amplitudes)
             means.append(channel_means)
             variances.append(channel_variances)
         return amplitudes, means, variances
 
-    def rescale_x(self):
-        xmin = {}
-        xmax = {}
-        for channel in self.channels:
-            for i, formatter in enumerate(channel.formatters[:-1]):
-                if not hasattr(formatter, 'category'):
-                    formatter.category = 'none'
-
-                x = channel.get_data()[0]
-                if formatter.category not in xmin:
-                    xmin[formatter.category] = np.min(x[:,i])
-                    xmax[formatter.category] = np.max(x[:,i])
-                else:
-                    xmin[formatter.category] = min(xmin[formatter.category], np.min(x[:,i]))
-                    xmax[formatter.category] = max(xmax[formatter.category], np.max(x[:,i]))
-
-        for channel in self.channels:
-            offsets = []
-            scales = []
-            for i, formatter in enumerate(channel.formatters[:-1]):
-                offsets.append(xmin[formatter.category])
-                scales.append(1000.0 / (xmax[formatter.category] - xmin[formatter.category]))
-            channel.set_x_scaling(offsets, scales)
-
-    def _to_kernel(self):
+    def plot(self, pred=None, title=None, figsize=None, legend=True, transformed=False):
         """
-        Return the data vectors in the format as used by the kernels.
-
-        Returns:
-            numpy.ndarray: X data of shape (n,2) where X[:,0] contains the channel indices and X[:,1] the X values.
-            numpy.ndarray: Y data.
-
-        Examples:
-            >>> x, y = dataset._to_kernel()
-        """
-        x = [channel.X[channel.mask] for channel in self.channels]
-        y = [channel.Y[channel.mask] for channel in self.channels]
-
-        chan = [i * np.ones(len(x[i])) for i in range(len(x))]
-        chan = np.concatenate(chan).reshape(-1, 1)
-        
-        x = np.concatenate(x)
-        x = np.concatenate((chan, x), axis=1)
-        if y == None:
-            return x
-
-        y = np.concatenate(y).reshape(-1, 1)
-        return x, y
-
-    def _to_kernel_prediction(self):
-        """
-        Return the prediction input vectors in the format as used by the kernels.
-
-        Returns:
-            numpy.ndarray: X data of shape (n,2) where X[:,0] contains the channel indices and X[:,1] the X values.
-
-        Examples:
-            >>> x = dataset._to_kernel_prediction()
-        """
-        x = [channel.X_pred for channel in self.channels]
-
-        chan = [i * np.ones(len(x[i])) for i in range(len(x))]
-        chan = np.concatenate(chan).reshape(-1, 1)
-        if len(chan) == 0:
-            return np.array([]).reshape(-1, 1)
-
-        x = np.concatenate(x)
-        x = np.concatenate((chan, x), axis=1)
-        return x
-
-    def _from_kernel_prediction(self, name, mu, var):
-        """
-        Returns the predictions from the format as used by the kernels. The prediction is stored in the Data class by the given name.
+        Plot the data including removed observations, latent function, and predictions for each channel.
 
         Args:
-            name (str): Name to store the prediction under.
-            mu (numpy.ndarray): Y mean prediction of shape (m*n(m)), i.e. a flat array of n(m) data points per channel m.
-            var (numpy.ndarray): Y variance prediction of shape (m*n(m)), i.e. a flat array of n(m) data points per channel m.
-
-        Examples:
-            >>> x = np.array([0.0, 1.0, 2.0, 3.0, 4.0])
-            >>> mu, var = model.model.predict_f(x)
-            >>> dataset._from_kernel_prediction('MOSM', mu, var)
-        """
-        N = [len(channel.X_pred) for channel in self.channels]
-        if len(mu) != len(var) or sum(N) != len(mu):
-            raise ValueError("prediction mu or var different length from prediction x")
-
-        i = 0
-        for idx in range(len(self.channels)):
-            self.channels[idx].Y_mu_pred[name] = np.squeeze(mu[i:i+N[idx]])
-            self.channels[idx].Y_var_pred[name] = np.squeeze(var[i:i+N[idx]])
-            i += N[idx]
-
-    def copy(self):
-        """
-        Make a deep copy of DataSet.
-
-        Returns:
-            mogptk.dataset.DataSet
-
-        Examples:
-            >>> other = dataset.copy()
-        """
-        return copy.deepcopy(self)
-
-    def plot(self, title=None, figsize=None):
-        """
-        Plot each Data channel.
-
-        Args:
-            title (str, optional): Set the title of the plot.
+            pred (str): Specify model name to draw.
+            title (str): Set the title of the plot.
+            figsize (tuple): Set the figure size.
+            legend (boolean): Disable legend.
+            transformed (boolean): Display transformed Y data as used for training.
 
         Returns:
             matplotlib.figure.Figure: The figure.
             list of matplotlib.axes.Axes: List of axes.
 
         Examples:
-            >>> fig, axes = dataset.plot('Title')
+            >>> fig, axes = dataset.plot(title='Title')
         """
         if figsize is None:
-            figsize = (12, 2.5 * len(self))
+            figsize = (12, 3.0 * len(self.channels))
 
+        h = figsize[1]
+        fig, axes = plt.subplots(self.get_output_dims(), 1, figsize=figsize, squeeze=False, constrained_layout=True)
 
-        fig, axes = plt.subplots(self.get_output_dims(), 1, constrained_layout=True, squeeze=False, figsize=figsize)
-        if title != None:
-            fig.suptitle(title)
-
+        legends = {}
         for channel in range(self.get_output_dims()):
-            if channel == 0:
-                self.channels[channel].plot(ax=axes[channel,0], plot_legend=True)    
-            else:
-                self.channels[channel].plot(ax=axes[channel,0])
+            ax = self.channels[channel].plot(pred=pred, ax=axes[channel,0], transformed=transformed)
+            legend = ax.get_legend()
+            for text, handle in zip(legend.texts, legend.legendHandles):
+                if text.get_text() == "Training Points":
+                    handle = plt.Line2D([0], [0], ls='-', color='k', marker='.', ms=10, label='Training Points')
+                legends[text.get_text()] = handle
+            legend.remove()
 
+        legend_rows = (len(legends)-1)/5 + 1
+        if title is not None:
+            fig.suptitle(title, y=(h+0.2+0.4*legend_rows)/h, fontsize=18)
 
+        fig.legend(handles=legends.values(), loc="upper center", bbox_to_anchor=(0.5,(h-0.3+0.5*legend_rows)/h), ncol=5)
         return fig, axes
 
+    def plot_spectrum(self, title=None, method='ls', per=None, maxfreq=None, figsize=None, transformed=False):
+        """
+        Plot the spectrum for each channel.
+
+        Args:
+            title (str): Set the title of the plot.
+            method (list, str): Set the method to get the spectrum such as LS or BNSE.
+            per (list, str): Set the scale of the X axis depending on the formatter used, eg. per=5, per='day', or per='3D'.
+            maxfreq (list, float): Maximum frequency to plot, otherwise the Nyquist frequency is used.
+            figsize (tuple): Set the figure size.
+            transformed (boolean): Display transformed Y data as used for training.
+
+        Returns:
+            matplotlib.figure.Figure: The figure.
+            list of matplotlib.axes.Axes: List of axes.
+
+        Examples:
+            >>> fig, axes = dataset.plot_spectrum(title='Title', method='bnse')
+        """
+        if not isinstance(method, list):
+            method = [method] * len(self.channels)
+        if not isinstance(per, list):
+            per = [per] * len(self.channels)
+        if not isinstance(maxfreq, list):
+            maxfreq = [maxfreq] * len(self.channels)
+
+        if figsize is None:
+            figsize = (12, 3.0 * len(self.channels))
+
+        fig, axes = plt.subplots(self.get_output_dims(), 1, figsize=figsize, squeeze=False, constrained_layout=True)
+        if title != None:
+            fig.suptitle(title, fontsize=18)
+
+        for channel in range(self.get_output_dims()):
+            ax = self.channels[channel].plot_spectrum(method=method[channel], ax=axes[channel,0], per=per[channel], maxfreq=maxfreq[channel], transformed=transformed)
+        return fig, axes
